
# borg-assmiliator/Oma.pm: OMA assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# load our perl modules.
use Cwd;
use SOAP::Lite;

# setup the oma http location.
my $oma_http_host = "omabrowser.org";
my $oma_http_dir = "/All";
my $oma_http_file = "oma-uniprot.txt.gz";
my $oma_db_file = "oma-uniprot.txt";

# the global hash table.
my @oma_mapping;

# oma_get_db: downloads the database, if necessary.
sub oma_get_db {
  # get the arguments.
  my $cwd = getcwd ();

  # do we already have the oma file?
  print "checking for local copy of oma mappings.\n";
  return 1 if (-e $cwd . "/" . $oma_db_file);

  # don't redownload this large file.
  print "downloading a local copy of oma mappings.\n";
  unless (-e $cwd . "/" . $oma_http_file) {
    # download the file.
    http_get ($oma_http_host, $oma_http_dir . "/" . $oma_http_file,
              $cwd . "/") or die ("failed to download oma mappings");
  }

  # unpack the gzip file.
  print "unpacking local copy of oma mappings.\n";
  gunzip $cwd . "/" . $oma_http_file => $oma_http_file . ".u"
    or die ("failed to unpack " . $oma_http_file);

  # open the unzipped file.
  open (OMAGROUPS, "<", $cwd . "/" . $oma_http_file . ".u");

  # open the output file.
  open (OMAMAP, ">", $cwd . "/" . $oma_db_file);

  # process the unzipped file.
  while (my $line = <OMAGROUPS>) {
    # see if we can pull data from the line.
    if ($line =~ /^([^\t]*)\t([A-Z][0-9]{5}?).*$/) {
      # get the values.
      my $omaGroup = trim ($1);
      my $omaUID = trim ($2);

      # output the pairing.
      print OMAMAP $omaUID . "," . $omaGroup . "\n";
    }
  }

  # close the description file.
  close (OMAMAP);

  # close the unzipped file.
  close (OMAGROUPS);

  # remove the gzip file.
  unlink ($cwd . "/" . $oma_http_file);
  unlink ($cwd . "/" . $oma_http_file . ".u");
}

# oma_read_db: reads the oma table into a hash table.
sub oma_read_db {
  # get the arguments.
  my $cwd = getcwd ();

  # print a nice message.
  print "reading in local copy of oma mappings.\n";
  return 0 unless (-e $cwd . "/" . $oma_db_file);

  # open the description file.
  open (OMAMAP, "<", $cwd . "/" . $oma_db_file);

  # read the description file.
  while (my $line = <OMAMAP>) {
    # see if we can pull data from the line.
    if ($line =~ /^([^,]*),(.*)$/) {
      # get the values.
      my $omaUID = trim ($1);
      my $omaGroup = trim ($2);

      # add the pair to the hash table.
      push (@oma_mapping, $omaUID . " " . $omaGroup);
    }
  }

  # close the description file.
  close (OMAMAP);
  return 1;
}

# oma_get_group: parses the downloaded oma-uniprot mapping.
sub oma_get_group {
  # read in the argument.
  my $uid = shift (@_);
  return unless defined ($uid);

  # loop through the mappings.
  foreach my $mapping (@oma_mapping) {
    # try to parse the mapping.
    if ($mapping =~ /^([^ ]*) (.*)$/) {
      # get the parsed values.
      my $omaUID = trim ($1);
      my $omaGroup = trim ($2);

      # check for identity.
      if ($omaUID eq $uid) {
        return $omaGroup;
      }
    }
  }

  # no match.
  return;
}

# oma_get_group_remote: tries to find the oma group for a uniprot id.
sub oma_get_group_remote {
  # get the arguments.
  my $cwd = getcwd ();
  my $uid = shift (@_);
  my $oma;

  # output the oma group.
  print "searching for oma group for uid " . $uid . "... ";

  # set up the soap service.
  my $service = SOAP::Lite
    -> uri('urn:/OMA')
    -> proxy('http://omabrowser.org/cgi-bin/soap.pl');

  # run the function.
  my $result = $service->GetEntry ($uid);
  $oma = $result->valueof ('//GetEntryResponse/OMAGroup');

  # finish the statement.
  if (defined ($oma)) {
    # the result.
    print $oma . ".\n";
  }
  else {
    # newline.
    print "none found.\n";
  }

  # return the oma group.
  return $oma;
}

# cause its a module.
return 1;
