
# borg-assmiliator/String.pm: STRING database assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# setup the string file locations.
my $string_file;
my $string_http_host = "string-db.org";
my $string_http_resource = "/api/psi-mi/interactions?identifier=";
my $string_http_file = "interactions?identifier=";
my $string_db_dir = "string-db";

# declare the string hash.
my %string_link_hash;
my %string_score_hash;
my %string_orig_hash;

# string_download: downloads a string search result from the interwebs.
sub string_download {
  # get the arguments.
  my $cwd = getcwd ();
  my $stringID = shift (@_);

  # does the string db dir exist?
  mkdir ($cwd . "/" . $string_db_dir)
    unless (-e $cwd . "/" . $string_db_dir);

  # check if we have the file.
  $string_file = $cwd . "/" . $string_db_dir . "/" . $stringID;
  unless (-e $string_file) {
    # download the file.
    http_get ($string_http_host, $string_http_resource . $stringID,
              $cwd . "/");

    # move the file into the correct location.
    rename ($cwd . "/" . $string_http_file . $stringID, $string_file);
  }
}

# string_parse_single: adds a single instance to the hash file.
sub string_parse_single {
  # get the parameters.
  my $stringFile = shift (@_);
  my $xmlError = 0;

  # load the xml parser and grab its contents.
  my $xmlParser = XML::LibXML->new ();
  my $xmlTree = $xmlParser->parse_file ($string_db_dir . "/" . $stringFile);
  my $xmlRoot = $xmlTree->documentElement ();
  my @xmlMetas = $xmlTree->getElementsByTagName ("meta");

  # loop through the meta tags.
  foreach my $meta (@xmlMetas) {
    # does this tag represent an error?
    if (defined ($meta->getAttribute ("name")) and 
        $meta->getAttribute ("name") eq "error") {
      # yeah, an error.
      $xmlError = 1;
    }
  }

  # ok, parse the file.
  unless ($xmlError) {
    # get the interactors in the xml file.
    my @interactors = $xmlTree->getElementsByTagName ("interactor");
    my @interactions = $xmlTree->getElementsByTagName ("interaction");

    # loop through the interactions.
    foreach my $interaction (@interactions) {
      # get the interaction id.
      my $interactionID = $interaction->getAttribute ("id");

      # only parse interactions with experimentRef.id=0 (composite)
      my @exptRefs = $interaction->getElementsByTagName ("experimentRef");
      my $interactionType = $exptRefs[0]->textContent;
      next unless ($interactionType eq "0");

      # get the interaction participant ids.
      my @participants = $interaction->getElementsByTagName ("interactorRef");
      my $partA = $participants[0]->textContent;
      my $partB = $participants[1]->textContent;

      # get the interaction score.
      my @stringScores = $interaction->getElementsByTagName ("value");
      my $stringScore = $stringScores[0]->textContent;

      # loop through the participants.
      my $uniprotRef;
      my $uniprotA;
      my $uniprotB;
      foreach my $interactor (@interactors) {
        # get the interactor id.
        my $interactorID = $interactor->getAttribute ("id");
        my @secondaries = $interactor->getElementsByTagName ("secondaryRef");

        # find the uniprot id.
        $uniprotRef = "";
        foreach my $secondary (@secondaries) {
          # is this the uniprot accessor id?
          if ($secondary->getAttribute ("db") eq "Ensembl_UniProt_AC" ||
              $secondary->getAttribute ("db") eq "BLAST_UniProt_AC") {
            # we've found it. get the id.
            $uniprotRef = $secondary->getAttribute ("id");

            # is this a valid uniprot id?
            if (uniprot_is_valid ($uniprotRef)) {
              # yes, yank and check if its reviewed.
              uniprot_yank_from_db ($uniprotRef);
              if (uniprot_is_reviewed ($uniprotRef)) {
                # valid, yanked and reviewed. cool.
                last;
              }
            }
          }
        }

        # is this the right interactor?
        if ($interactorID eq $partA) {
          # set the uniprot value.
          $uniprotA = $uniprotRef;
        }
        elsif ($interactorID eq $partB) {
          # set the uniprot value.
          $uniprotB = $uniprotRef;
        }
      }

      # continue if we found no uniprot for either interactor.
      next unless defined ($uniprotA) and defined ($uniprotB);
      next if ($uniprotA eq "" or $uniprotB eq "");

      # make sure they exist. does this speed things up?
      next unless uniprot_is_valid ($uniprotA);
      next unless uniprot_is_valid ($uniprotB);

      # make sure both are reviewed.
      uniprot_yank_from_db ($uniprotA);
      next unless uniprot_is_reviewed ($uniprotA);
      uniprot_yank_from_db ($uniprotB);
      next unless uniprot_is_reviewed ($uniprotB);

      # make the string linkage.
      my $linkage = $uniprotA . ":" . $uniprotB;

      # add the linkage to the link hash.
      $string_link_hash{$linkage} = 1;

      # add the score to the score hash.
      if (defined ($string_score_hash{$linkage})) {
        # add only higher scores.
        $string_score_hash{$linkage} = $stringScore
          if ($stringScore > $string_score_hash{$linkage});
      }
      else {
        # we must have defined values.
        $string_score_hash{$linkage} = $stringScore;
      }

      # determine whether the two uniprots are originals.
      unless (defined ($string_orig_hash{$linkage})) {
        $string_orig_hash{$linkage} = 1
          if (uniprot_is_original ($uniprotA)
          and uniprot_is_original ($uniprotB));
      }
    }
  }
}

# string_build_hash: builds the hash for dumping into the database.
sub string_build_hash {
  # get the working dir.
  my $cwd = getcwd ();

  # build the list of string files.
  opendir (STRINGS, $string_db_dir)
    or die ("failed to open " . $string_db_dir);
  my @stringFiles = readdir (STRINGS);
  closedir (STRINGS);

  # output a quick info message.
  print "ok... parsing " . @stringFiles .
        " downloaded string interaction files.\n";

  # loop through the string files.
  foreach my $stringFile (@stringFiles) {
    # dont process the special files.
    next if ($stringFile eq "." or $stringFile eq "..");

    # parse the string file.
    string_parse_single ($stringFile);
  }
}

# string_dump_hash: dumps the hash into the database.
sub string_dump_hash {
  # loop through the linkage hash.
  foreach my $linkage (keys (%string_link_hash)) {

    # get the actors.
    my @actors = split (/:/, $linkage);

    # get the score.
    my $score = $string_score_hash{$linkage};

    # generate the originality string.
    my $original = "false";
    $original = "true" if defined ($string_orig_hash{$linkage});

    # add this linkage to the database.
    sql_insert ("string", $actors[0], $actors[1], $score, $original);
  }
}

# cause its a module.
return 1;
