#!/usr/bin/perl

# borg-assmiliator/borg.pl: main assimilator perl script
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# load our perl modules.
use BORG::Core;
use BORG::SQL;
use BORG::Sources;
use BORG::KEGG;
use BORG::Uniprot;
use BORG::eggNOG;
use BORG::Pfam;
use BORG::PDB;
use BORG::String;
use BORG::Dip;
use BORG::Oma;
use BORG::Binding;

# setup the kegg file parsers.
use BORG::KeggParser;
use BORG::KeggPathwayParser;
use BORG::KeggDiseaseParser;

# back to the main script.
package main;

# are the proteins we're running on indicated in pancreatic cancer?
$BORG::Core::pdacIndicator = "false";

# get the cwd.
my $cwd = getcwd ();

# print the greeting.
print_greeting ();

# download the uniprot database to a local file.
uniprot_get_db () or die ("failed to download uniprot db");

# build the list of aliases.
uniprot_build_aliases () or die ("failed to build uniprot aliases table");
uniprot_check_aliases_table () or die ("failed to check aliases table");

# build the list of sources.
sources_build_table () or die ("failed to build sources table");

# download the eggnog description file.
eggnog_get_db () or die ("failed to download eggnog db");
eggnog_read_db () or die ("failed to read in local eggnog db");

# download the pfam description file.
pfam_get_db () or die ("failed to download pfam db");
pfam_read_db () or die ("failed to read in local pfam db");

# download the oma mappings file.
oma_get_db () or die ("failed to download oma mappings");
oma_read_db () or die ("failed to read in local oma mappings");

# output a message.
print "checking for uniprot files from initial list.\n";

# read in the uniprot.txt file.
uniprot_list_read ();

# connect to the mysql database.
sql_connect ();

# ensure the tables exist.
sql_check_tables
 ("aliases",
  "sources",
  "uniprot",
  "genes",
  "diseases",
  "keywords",
  "seqsims",
  "GO",
  "pdbProteins",
  "pdbLigands",
  "kegg",
  "keggPathways",
  "keggDiseases",
  "ensembl",
  "string",
  "dip",
  "pfam",
  "eggNOG",
  "OMA",
  "binding");

# check if the uniprot table needs updating.
print "checking for a complete set of uniprot tables.\n";
unless (sql_is_table_nonempty ("uniprot")) {
  # output a quick info message.
  print "ok... parsing yanked uniprot files.\n";

  # build the mysql tables.
  uniprot_build_tables ();
}

# check if we need to update the ligands table.
print "checking for a complete set of pdb tables.\n";
unless (sql_is_table_nonempty ("pdbLigands")) {
  # build the list of excluded pdb ligands.
  pdb_build_exclusion_hash ();

  # build the mysql tables.
  pdb_build_tables ();
}

# check if we need to update the kegg tables.
print "checking for a complete set of kegg tables.\n";
unless (sql_is_table_nonempty ("keggPathways")) {
  # build the mysql tables.
  kegg_build_tables ();
}

# check for a complete string table.
print "checking for a complete string table.\n";
unless (sql_is_table_nonempty ("string")) {
  # build the hash.
  print "ok... building string database hash table.\n";
  string_build_hash ();

  # dump the hash.
  print "dumping string hash table into database.\n";
  string_dump_hash ();
}

# check for a complete dip table.
print "checking for a complete dip table.\n";
unless (sql_is_table_nonempty ("dip")) {
  # build the hash.
  dip_build_hash ();

  # dump the hash.
  dip_dump_hash ();
}

# check for a complete binding table.
print "checking for a complete binding table.\n";
unless (sql_is_table_nonempty ("binding")) {
  # build the hash.
  binding_build_hash ();

  # dump the hash.
  binding_dump_hash ();
}

# disconnect from the mysql database.
sql_disconnect ();

# print a message so we know its done.
print "all done!\n";
