
# borg-assmiliator/KEGG.pm: KEGG assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# load our perl modules.
use Cwd;

# declare kegg strings here.
my $kegg_file;
my $kegg_dir = "/kegg/";
my $kegg_http_host = "www.genome.jp";

# kegg_http_get: downloads a kegg gene file.
sub kegg_http_get {
  # get the kegg id.
  my $cwd = getcwd ();
  my $kid = shift;

  # make sure the directory exists.
  ensure_directory ($cwd . $kegg_dir);

  # set up the kegg filename.
  $kegg_file = $kid . ".txt";
  $kegg_file =~ s/:/_/g;

  # make sure we arent redownloading the same file.
  unless (-e $cwd . $kegg_dir . $kegg_file) {
    # download the file.
    http_get ($kegg_http_host, "/dbget-bin/www_bget?" . $kid,
              $cwd . $kegg_dir);

    # rename the file.
    rename ($kegg_dir . "www_bget?" . $kid,
            $cwd . $kegg_dir . $kegg_file);
  }
}

# kegg_build_tables: builds the kegg diseases and pathways tables.
sub kegg_build_tables {
  # get the vars.
  my $cwd = getcwd ();

  # build the list of kegg files.
  opendir (KEGGS, $cwd . $kegg_dir)
    or die ("failed to open " . $cwd . $kegg_dir);
  my @keggFiles = readdir (KEGGS);
  closedir (KEGGS);

  # output a quick info message.
  print "ok... parsing " . @keggFiles . " downloaded kegg info files.\n";

  # loop through the kegg files.
  foreach my $kegg_file (@keggFiles) {
    # dont process the special files.
    next if ($kegg_file eq "." or $kegg_file eq "..");

    # get the kegg id string.
    my $keggId = $kegg_file;
    $keggId =~ s/\.txt//g;
    $keggId =~ s/_/:/g;

    # load the html parser and grab its elements.
    my $keggParser = BORG::KeggParser->new ();
    $keggParser->parse_file ($cwd . $kegg_dir . $kegg_file);

    # add kegg pathways to the "keggPathways" table.
    foreach my $pathway (@BORG::KeggParser::pathways) {
      # see if we should download the kegg pathway file.
      my $keggPathwayFile = "path_" . $pathway . ".txt";
      unless (-e $cwd . $kegg_dir . $keggPathwayFile) {
        # download the file.
        http_get ($kegg_http_host, "/dbget-bin/www_bget?path:" . $pathway,
                  $cwd . $kegg_dir);

        # rename the file.
        rename ($cwd . $kegg_dir . "www_bget?path:" . $pathway,
                $cwd . $kegg_dir . $keggPathwayFile);
      }

      # set up a kegg pathway parser.
      my $keggPathwayParser = BORG::KeggPathwayParser->new ();
      $keggPathwayParser->parse_file ($cwd . $kegg_dir . $keggPathwayFile);
      my $pathwayDesc = $BORG::KeggPathwayParser::pathwayName;
      next unless defined ($pathwayDesc);
      $pathwayDesc =~ s/ - Homo sapiens \(human\)//g;

      # add the entry into the kegg pathways table.
      sql_insert ("keggPathways",
                  sql_validate ($keggId),
                  sql_validate ($pathway),
                  sql_validate ($pathwayDesc));

      # clear the pathway value.
      undef $BORG::KeggPathwayParser::pathwayName;
    }

    # add kegg diseases to the "keggDiseases" table.
    foreach my $disease (@BORG::KeggParser::diseases) {
      # see if we should download the kegg disease file.
      my $keggDiseaseFile = "ds_" . $disease . ".txt";
      unless (-e $cwd . $kegg_dir . $keggDiseaseFile) {
        # download the file.
        http_get ($kegg_http_host, "/dbget-bin/www_bget?ds:" . $disease,
                  $cwd . $kegg_dir);

        # rename the file.
        rename ($cwd . $kegg_dir . "www_bget?ds:" . $disease,
                $cwd . $kegg_dir . $keggDiseaseFile);
      }

      # set up a kegg disease parser.
      my $keggDiseaseParser = BORG::KeggDiseaseParser->new ();
      $keggDiseaseParser->parse_file ($cwd . $kegg_dir . $keggDiseaseFile);
      my $diseaseDesc = $BORG::KeggDiseaseParser::diseaseName;

      # add the entry into the kegg diseases table.
      sql_insert ("keggDiseases",
                  sql_validate ($keggId),
                  sql_validate ($disease),
                  sql_validate ($diseaseDesc));

      # clear the disease value.
      undef $BORG::KeggDiseaseParser::diseaseName;
    }

    # clear the arrays.
    @BORG::KeggParser::pathways = ();
    #delete $BORG::KeggParser::pathways[0 .. $#BORG::KeggParser::pathways];
    @BORG::KeggParser::diseases = ();
    #delete $BORG::KeggParser::diseases[0 .. $#BORG::KeggParser::diseases];
  }
}

# cause its a module.
return 1;
