
# borg-assmiliator/Uniprot.pm: UniProtKB assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# load our perl modules.
use Cwd;
use Net::FTP;
use IO::Uncompress::Gunzip qw(gunzip $GunzipError);

# setup the uniprot ftp location.
my $uniprot_ftp_host = "ftp.uniprot.org";
my $uniprot_ftp_dir = "pub/databases/uniprot/current_release/knowledgebase/complete";
my $uniprot_ftp_file = "uniprot_sprot.dat.gz";
my $uniprot_db_file = "uniprot-db.dat";
my $uniprot_ac_file = "uniprot-ac.lst";
my $uniprot_db_dir = "uniprot-db";

# the list of uniprot identifiers to parse data for.
my %uniprotIDs;

# uniprot_get_db_dir: returns the database directory.
sub uniprot_get_db_dir {
  # get the working dir.
  my $cwd = getcwd ();

  # return the database directory.
  return $cwd . "/" . $uniprot_db_dir . "/";
}

# uniprot_get_db: downloads the database, if necessary.
sub uniprot_get_db {
  # get the arguments.
  my $cwd = getcwd ();

  # do we already have the uniprot file?
  print "checking for local copy of uniprot database.\n";
  return 1 if (-e $cwd . "/" . $uniprot_db_file);

  # don't redownload this large file.
  print "downloading a local copy of uniprot database.\n";
  unless (-e $cwd . "/" . $uniprot_ftp_file) {
    # setup the ftp connection.
    my $ftp = Net::FTP->new ($uniprot_ftp_host, Debug => 0)
      or die ("failed to connect to " . $uniprot_ftp_host);

    # login.
    $ftp->login ("anonymous", "anonymous")
      or die ("failed to login to " . $uniprot_ftp_host);

    # change to the proper directory.
    $ftp->cwd ($uniprot_ftp_dir)
      or die ("failed to cwd into " . $uniprot_ftp_dir);

    # get the uniprot file.
    $ftp->binary ();
    $ftp->get ($uniprot_ftp_file, $cwd . "/" . $uniprot_ftp_file)
      or die ("failed to get " . $uniprot_ftp_file);
  }

  # unpack the gzip file.
  print "unpacking local copy of uniprot database.\n";
  gunzip $cwd . "/" . $uniprot_ftp_file => $uniprot_db_file
    or die ("failed to unpack " . $uniprot_ftp_file);

  # remove the gzip file.
  unlink ($cwd . "/" . $uniprot_ftp_file);

  # unpack the unpacked file. :P
  uniprot_unpack_db ();
}

# uniprot_unpack_db: yanks all uniprot files from the uniprot database.
sub uniprot_unpack_db {
  # get the arguments.
  my $cwd = getcwd ();

  # does the uniprot db dir exist?
  mkdir ($cwd . "/" . $uniprot_db_dir)
    unless (-e $cwd . "/" . $uniprot_db_dir);

  # setup vars for reading the db.
  my $writing = 0;
  my $prev = "";
  my $uid;

  # output a message.
  print "ensuring the local copy of uniprot database is unpacked.\n";

  # open the uniprot db file.
  open (UNIPROTDB, "<", $cwd . "/" . $uniprot_db_file);

  # read the file.
  while (my $line = <UNIPROTDB>) {
    # are we writing?
    if ($writing == 1) {
      # write the current line.
      print YANKED $line;

      # are we done writing?
      if ($line =~ /^\/\/$/) {
        # end writing.
        close (YANKED);
        $writing = 0;
        undef $uid;
      }
    }
    else {
      # is this an identifier line?
      if ($line =~ /^ID */) {
        # start writing the "previous lines string".
        $prev = $line;
      }
      elsif ($line =~ /^AC   (.*);$/) {
        # get the first accessor.
        my @accessors = split ("; ", $1);

        # see if we should open the output file.
        unless (defined ($uid)) {
          # set the uid string and open the file.
          $uid = $accessors[0];
          unless (-e $cwd . "/" . $uniprot_db_dir . "/" . $uid) {
            open (YANKED, ">", $cwd . "/" . $uniprot_db_dir . "/" . $uid);
            print YANKED $prev;
            print YANKED $line;
            $writing = 1;

            # link the other accessors to the requested one.
            foreach my $accessor (@accessors) {
              # dont relink already linked accessors.
              unless (-e $cwd . "/" . $uniprot_db_dir . "/" . $accessor) {
                # make the symbolic link.
                symlink ($uid, $cwd . "/" . $uniprot_db_dir . "/" . $accessor);
              }
            }
          }
        }
      }
      else {
        # add to the previous line value.
        $prev .= $line;
      }
    }
  }

  # close the uniprot db file.
  close (UNIPROTDB);
}

# uniprot_yank_from_db: yanks a uniprot txt file from the uniprot database.
sub uniprot_yank_from_db {
  # get the arguments.
  my $cwd = getcwd ();
  my $uid = shift (@_);

  # does the uniprot db dir exist?
  mkdir ($cwd . "/" . $uniprot_db_dir)
    unless (-e $cwd . "/" . $uniprot_db_dir);

  # setup vars for reading the db.
  my $filename = $cwd . "/" . $uniprot_db_dir . "/" . $uid;
  my $writing = 0;
  my $prev = "";

  # bail if the uniprot file already exists.
  return 1 if (-e $filename);

  # output a message.
  print "yanking " . $uid . " from local copy of uniprot database.\n";

  # open the uniprot db file.
  open (UNIPROTDB, "<", $cwd . "/" . $uniprot_db_file);

  # read the file.
  while (my $line = <UNIPROTDB>) {
    # are we writing?
    if ($writing == 1) {
      # write the current line.
      print YANKED $line;

      # are we done writing?
      if ($line =~ /^\/\/$/) {
        # end writing.
        close (UNIPROTDB);
        close (YANKED);
        $writing = 0;
        return 1;
      }
    }
    else {
      # is this an identifier line?
      if ($line =~ /^ID */) {
        # start writing the "previous lines string".
        $prev = $line;
      }
      elsif ($line =~ /^AC   (.*);$/) {
        # get the first accessor.
        my @accessors = split ("; ", $1);

        # is the uid we asked for in the list?
        if (grep (/$uid/i, @accessors)) {
          # start writing the file.
          open (YANKED, ">", $filename);
          print YANKED $prev;
          print YANKED $line;
          $writing = 1;

          # link the other accessors to the requested one.
          foreach my $accessor (@accessors) {
            # dont relink already linked accessors.
            unless (-e $cwd . "/" . $uniprot_db_dir . "/" . $accessor) {
              # make the symbolic link.
              symlink ($uid, $cwd . "/" . $uniprot_db_dir . "/" . $accessor);
            }
          }
        }
        else {
          # add to the previous line value.
          $prev .= $line;
        }
      }
      else {
        # add to the previous line value.
        $prev .= $line;
      }
    }
  }

  # close the uniprot db file.
  close (UNIPROTDB);

  # write a "blank" uniprot entry.
  open (YANKED, ">", $filename);
  print YANKED "BORG_ERROR_UNIPROT_IS_INVALID\n";
  close (YANKED);
}

# uniprot_is_valid: returns whether a given uniprot id exists in the database.
sub uniprot_is_valid {
  # get the arguments.
  my $cwd = getcwd ();
  my $uid = shift (@_);

  # if the file exists, return ok.
  return 0 unless (-e $cwd . "/" . $uniprot_db_dir . "/" . $uid);

  # open, read the first line of the file, close.
  open (UNIPROTFILE, "<", $cwd . "/" . $uniprot_db_dir . "/" . $uid);
  my $line1 = <UNIPROTFILE>;
  close (UNIPROTFILE);

  # see if we have an invalid message.
  return 0 if ($line1 =~ /BORG_ERROR_UNIPROT_IS_INVALID/);

  # yup, valid.
  return 1;
}

# uniprot_is_reviewed: returns whether or not a uniprot entry is reviewed.
sub uniprot_is_reviewed {
  # get the arguments.
  my $cwd = getcwd ();
  my $uid = shift (@_);

  # make sure the file exists for reading.
  # the expectation is that the file has been pre-yanked.
  unless (-e $cwd . "/" . $uniprot_db_dir . "/" . $uid) {
    # does not exist.
    print "uniprot entry " . $uid . " is not yanked.\n";
    return 0;
  }

  # open the file, read the first line.
  open (UNIPROTFILE, "<", $cwd . "/" . $uniprot_db_dir . "/" . $uid);
  my $line = <UNIPROTFILE>;
  close (UNIPROTFILE);

  # see if its Reviewed.
  if ($line =~ /^ID.*Reviewed;.*$/) {
    # we are.
    return 1;
  }

  # i guess not.
  print "uniprot entry " . $uid . " is unreviewed.\n";
  return 0;
}

# uniprot_build_aliases: builds a list of aliases from the uniprot list.
sub uniprot_build_aliases {
  # get the cwd.
  my $cwd = getcwd ();

  # output a message.
  print "checking for list of aliases from initial list.\n";

  # exit if the file exists.
  return 1 if (-e $cwd . "/aliases.txt");

  # fail if the file doesnt exist.
  return 0 unless (-e $cwd . "/uniprot.txt");

  # output a message.
  print "building a list of aliases from initial list.\n";

  # open the input and output files.
  open (UNIPROT, "<", $cwd . "/uniprot.txt");
  open (ALIASES, ">", $cwd . "/aliases.txt");

  # loop through the uniprot.txt file.
  while (my $line = <UNIPROT>) {
    # get the uid.
    my $uid = uc (trim ($line));
    my @aclines;

    # open the uniprot file in question.
    open (UNIPROTFILE, "<", $cwd . "/" . $uniprot_db_dir . "/" . $uid);

    # loop through the file.
    while (my $uline = <UNIPROTFILE>) {
      # see if the line is an accessor
      if ($uline =~ /^AC   (.*);$/) {
        # add the line to the list.
        push (@aclines, trim ($1));
      }
    }

    # close the uniprot file.
    close (UNIPROTFILE);

    # loop through the accessor lines.
    print ALIASES $uid . ": ";
    foreach my $acline (@aclines) {
      # output the line into the aliases file.
      $acline =~ s/\;//g;
      print ALIASES $acline . " ";
    }
    print ALIASES "\n";
  }

  # close the uniprot file.
  close (UNIPROT);
  close (ALIASES);

  # return.
  return 1;
}

# uniprot_check_aliases_table: checks for an aliases table.
sub uniprot_check_aliases_table {
  # get the cwd.
  my $cwd = getcwd ();

  # connect to the sql database.
  sql_connect ();

  # return if we have the table.
  return 1 if sql_is_table_nonempty ("aliases");

  # make sure the aliases file exists.
  die ("could not find the alias file") unless (-e $cwd . "/aliases.txt");

  # open the aliases file.
  open (ALIASES, "<", $cwd . "/aliases.txt");

  # loop through the aliases file.
  while (my $line = <ALIASES>) {
    # get the uid and aliases.
    if ($line =~ /^(.*): (.*)$/) {
      # make the vars.
      my $uid = uc ($1);
      my @aliases = split (/ /, $2);

      # add the line into the table.
      foreach my $alias (@aliases) {
        sql_insert ("aliases", $uid, $alias);
      }
    }
  }

  # disconnect from the sql database.
  sql_disconnect ();

  # close the aliases file.
  close (ALIASES);
}

# uniprot_list_read: reads the list of uniprot ids into a hash table.
sub uniprot_list_read {
  # get the cwd.
  my $cwd = getcwd ();

  # open the uniprot.txt file.
  open (UNIPROT, "<", $cwd . "/uniprot.txt");

  # read lines from the uniprot.txt file.
  while (my $uniprotID = <UNIPROT>) {
    # clean up the uniprot id.
    $uniprotID = uc (trim ($uniprotID));

    # yank the uniprot txt file.
    uniprot_yank_from_db ($uniprotID)
      or die ("failed to yank uniprot file " . $uniprotID);

    # increment the counter.
    $uniprotIDs{$uniprotID} = 1;
  }

  # close the uniprot.txt file.
  close (UNIPROT);
}

# uniprot_is_original: returns whether the uniprot is in the original list.
sub uniprot_is_original {
  # get the arguments.
  my $uid = shift (@_);

  # set up the sql query.
  my $q;
  $q="select orig from aliases where alias='".$uid."' or orig='".$uid."';";

  # return the number of results.
  return sql_num_query_results ($q);
}

my %uniprot_originals;
sub uniprot_is_original_OLD {
  # get the arguments.
  my $cwd = getcwd ();
  my $uid = shift (@_);

  # caching may help here.
  return 1 if defined ($uniprot_originals{$uid});

  # make sure the aliases file exists.
  die ("could not find the alias file") unless (-e $cwd . "/aliases.txt");

  # open the aliases file.
  open (ALIASES, "<", $cwd . "/aliases.txt");

  # loop through the aliases file.
  while (my $line = <ALIASES>) {
    # check the line.
    if ($line =~ /$uid/) {
      # yep.
      close (ALIASES);
      $uniprot_originals{$uid} = 1;
      return 1;
    }
  }

  # nope.
  close (ALIASES);
  return 0;
}

# uniprot_build_tables: parses the uniprot hash table and saves data.
sub uniprot_build_tables {
  # get the arguments.
  my $cwd = getcwd ();

  # loop through the uniprot xml files.
  foreach my $uniprotID (keys (%uniprotIDs)) {
    # the protein name and gene names.
    my $fullEntry;
    my $fullName;
    my @geneNames;

    # the sequences.
    my $sequencing = 0;
    my @seqLengths;
    my @sequences;
    my $sequence = "";

    # the function.
    my $functioning = 0;
    my $function = "";

    # the subcellular location.
    my $subcellularLocation;

    # the associated diseases.
    my $diseasing = 0;
    my @diseases;
    my $disease = "";

    # the associated keywords.
    my @keywords;
    my $keyword;

    # the associated sequence similarities.
    my @seqsims;
    my $seqsim;

    # any GO annotations.
    my @GOids;
    my @GOtypes;
    my @GOdescs;

    # the pdb protein ids and locations.
    my @PDBids;
    my @PDBlocs;

    # the kegg ids.
    my @KEGGids;

    # the ensembl ids.
    my @ENSTs;
    my @ENSPs;
    my @ENSGs;

    # the string ids. (just uniprots?)
    my @STRINGs;

    # the pfam ids.
    my @pfams;

    # the eggnog ids.
    my @eggNOGs;

    # skip unreviewed uniprot ids.
    next unless uniprot_is_reviewed ($uniprotID);

    # open the uniprot file.
    open (UNIPROT, "<", uniprot_get_db_dir () . $uniprotID);

    # loop through the file. let's parse it.
    while (my $line = <UNIPROT>) {
      # are we sequencing?
      if ($sequencing == 1) {
        # have we finished sequencing yet?
        if ($line =~ /^\/\/$/) {
          # yes. push the sequence to the array.
          $sequence =~ s/ //g;
          $sequence = rtrim ($sequence);
          push (@sequences, $sequence);
          $sequence = "";
          $sequencing = 0;
        }
        else {
          # concatenate to the sequence string.
          $sequence = $sequence . $line;
        }
      }
      else {
        # search for gene name fields.
        if ($line =~ /^ID   ([^ ]*).*$/) {
          # add the entry value.
          $fullEntry = $1;
        }
        elsif ($line =~ /^DE   RecName: Full=([^;]*);.*$/) {
          # add the name value.
          $fullName = $1;
        }
        elsif ($line =~ /^GN   Name=([^;]*);.*$/) {
          # add the gene to the array.
          push (@geneNames, $1);
        }
        elsif ($line =~ /^SQ   SEQUENCE   (...) AA;.*$/) {
          # set the sequence length.
          push (@seqLengths, $1);

          # tell that we are sequencing.
          $sequencing = 1;
        }
        elsif ($line =~ /^KW   (.*)$/) {
          # take ends off of the keywords.
          $keyword = $1;
          $keyword =~ s/[;.]+$//;
          push (@keywords, split (/; /, $keyword));
        }
        elsif ($line =~ /^DR   ([^;]*); (.*)\.$/) {
          # set the variable names.
          my $db = $1;
          my @data = split (/; /, $2);

          # which database are we referencing?
          if ($db eq "GO") {
            # figure the type of GO term.
            my @gotype = split (/:/, $data[1]);

            # parse the GO id.
            push (@GOids, $data[0]);
            push (@GOtypes, $gotype[0]);
            push (@GOdescs, $gotype[1]);
          }
          elsif ($db eq "PDB") {
            # only use x-ray and nmr structures.
            if ($data[1] eq "X-ray" or $data[1] eq "NMR") {
              # parse in the PDB id.
              $data[3] =~ s/\.$//;
              push (@PDBids, $data[0]);
              push (@PDBlocs, $data[3]);

              # ensure we have the pdb ligand info file.
              pdb_download_ligands ($data[0]);
            }
          }
          elsif ($db eq "KEGG") {
            # add the KEGG id to the array.
            push (@KEGGids, $data[0]);

            # ensure we have the kegg gene file.
            kegg_http_get ($data[0]);
          }
          elsif ($db eq "Ensembl") {
            # add the Ensembl ids to the array.
            push (@ENSTs, $data[0]);
            push (@ENSPs, $data[1]);
            push (@ENSGs, $data[2]);
          }
          elsif ($db eq "STRING") {
            # add the STRING id to the array.
            push (@STRINGs, $data[0]);

            # ensure we have the string file.
            string_download ($data[0]);
          }
          elsif ($db eq "Pfam") {
            # add the pfam family to the array.
            push (@pfams, $data[0]);
          }
          elsif ($db eq "eggNOG") {
            # add the eggNOG data to the array.
            push (@eggNOGs, $data[0]);
          }
        }
        elsif ($line =~ /^CC   -!- ([^:]*): (.*)$/) {
          # are we beginning a function block?
          if ($1 eq "FUNCTION") {
            # yes.
            $functioning = 1;
            $function = $2;
          }
          else {
            # nope.
            $functioning = 0;
          }

          # sequence similarity.
          if ($1 eq "SIMILARITY") {
            # sequence similarity.
            $seqsim = trim ($2);
            $seqsim =~ s/^[Cc]ontains [0-9]* //g;
            $seqsim =~ s/^[Bb]elongs to the //g;
            $seqsim =~ s/s\.$//g;
            $seqsim =~ s/\.$//g;
            push (@seqsims, $seqsim);
          }

          # is this a subcellular location block?
          if ($1 eq "SUBCELLULAR LOCATION") {
            # add the first line.
            $subcellularLocation = $2;
            $subcellularLocation =~ s/\.$//;
          }

          # is this a disease block?
          if ($1 eq "DISEASE") {
            # are we already diseasing?
            if ($diseasing == 1) {
              # stop the old disease.
              push (@diseases, $disease);
              $diseasing = 0;
              $disease = "";
            }

            # add the first line.
            $diseasing = 1;
            $disease = $2;
          }
          else {
            # are we currently diseasing?
            if ($diseasing == 1) {
              # stop the old disease.
              push (@diseases, $disease);
              $diseasing = 0;
              $disease = "";
            }

            # nope.
            $diseasing = 0;
          }
        }
        elsif ($line =~ /^CC       (.*)$/) {
          # are we parsing a function block?
          if ($functioning == 1) {
            # add to the function string.
            $function = $function . " " . $1;
          }
          elsif ($diseasing == 1) {
            # add to the disease string.
            $disease = $disease . " " . $1;
          }
        }
      }
    }

    # close the uniprot file.
    close (UNIPROT);

    # check for null fields.
    $fullEntry = sql_validate ($fullEntry);
    $fullName = sql_validate ($fullName);
    $sequence = sql_validate ($sequences[0]);
    my $residues = sql_validate ($seqLengths[0]);
    $function = sql_validate ($function);
    $subcellularLocation = sql_validate ($subcellularLocation);

    # take the retarded linebreaks out of the sequence.
    $sequence =~ s/\s//g;
    
    # add this entry to the "uniprot" table.
    print $uniprotID . ": ";
    sql_insert ("uniprot",
                $uniprotID,
                $fullEntry, 
                $fullName,
                $sequence,
                $residues,
                $function,
                $subcellularLocation,
                $BORG::Core::pdacIndicator);

    # loop through the genes.
    print @geneNames . "g " if (@geneNames);
    foreach my $gene (@geneNames) {
      sql_insert ("genes", $uniprotID, sql_validate ($gene));
    }

    # loop through the diseases.
    print @diseases . "d " if (@diseases);
    foreach $disease (@diseases) {
      sql_insert ("diseases", $uniprotID, sql_validate ($disease));
    }

    # loop through the keywords.
    print @keywords . "k " if (@keywords);
    foreach $keyword (@keywords) {
      sql_insert ("keywords", $uniprotID, sql_validate ($keyword));
    }

    # loop through the sequence similarities.
    print @seqsims . "s " if (@seqsims);
    foreach $seqsim (@seqsims) {
      sql_insert ("seqsims", $uniprotID, sql_validate ($seqsim));
    }

    # loop through the GO annotations.
    print @GOids . "G " if (@GOids);
    for (my $iter = 0; $iter < @GOids; $iter++) {
      sql_insert ("GO", $uniprotID,
                  sql_validate ($GOids[$iter]),
                  sql_validate ($GOtypes[$iter]),
                  sql_validate ($GOdescs[$iter]));
    }

    # loop through the pdb protein ids.
    print @PDBids . "P " if (@PDBids);
    foreach my $PDBid (@PDBids) {
      sql_insert ("pdbProteins", $uniprotID, sql_validate ($PDBid));
    }

    # loop through the kegg ids.
    print @KEGGids . "K " if (@KEGGids);
    foreach my $KEGGid (@KEGGids) {
      sql_insert ("kegg", $uniprotID, sql_validate ($KEGGid));
    }

    # loop through the ensembl ids.
    print @ENSPs . "E " if (@ENSPs);
    for (my $iter = 0; $iter < @ENSPs; $iter++) {
      sql_insert ("ensembl", $uniprotID,
                  sql_validate ($ENSTs[$iter]),
                  sql_validate ($ENSPs[$iter]),
                  sql_validate ($ENSGs[$iter]));
    }

    # loop through the pfam ids.
    print @pfams . "F " if (@pfams);
    foreach my $pfam (@pfams) {
      sql_insert ("pfam", $uniprotID,
                  sql_validate ($pfam),
                  sql_validate (pfam_get_desc ($pfam)));
    }

    # loop through the eggNOG ids.
    print @eggNOGs . "N " if (@eggNOGs);
    foreach my $eggNOG (@eggNOGs) {
      sql_insert ("eggNOG", $uniprotID,
                  sql_validate ($eggNOG),
                  sql_validate (eggnog_get_desc ($eggNOG)));
    }

    # try to get an oma group for this uniprot file.
    my $omaGroup = oma_get_group ($uniprotID);
    if (defined ($omaGroup) && $omaGroup ne 0) {
      # add the entry into the database.
      print "O";
      sql_insert ("OMA", $uniprotID, sql_validate ($omaGroup));
    }

    # print a newline.
    print "\n";
  }
}

# cause its a module.
return 1;
